package cn.ecpark.service.usergw.config;

import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigChangeListener;
import com.ctrip.framework.apollo.ConfigService;
import com.ctrip.framework.apollo.enums.ConfigSourceType;
import com.ctrip.framework.apollo.model.ConfigChangeEvent;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ConfigurableApplicationContext;

import cn.ecpark.service.usergw.biz.events.ConfigRefresh;

// @EnableApolloConfig("gateway.yml")
// @ConditionalOnProperty(prefix="app", value="id", matchIfMissing=true)
// @Configurable

public class ConfigApollo implements ConfigChangeListener {

    
    private static final org.slf4j.Logger log =
    org.slf4j.LoggerFactory.getLogger(ConfigApollo.class);

    // @ApolloConfig("gateway.yml")
    private Config config;
    @Autowired
    private ApplicationContext appContext;

    @Value("${apollo.meta:}")
    private String apolloMeta;

    @Value("${app.id:}")
    private String appID;

    @Value("${apollo.namespace:}")
    private String apolloNamespace;

    private int tryConnect;

    public ConfigApollo() {
        this.tryConnect = 1;
    }

    public void Connect() {
        if (this.isConnected() && this.tryConnect > 0) {
            this.tryConnect--;
            ConfigSourceType stype = null;
            if (apolloNamespace.equals("")) {
                String[] tryExamples = { "gateway.yml", "gateway.yaml", "Gateway.yml", "Gateway.yaml" };
                for (String ns : tryExamples) {
                    apolloNamespace = ns;
                    config = ConfigService.getConfig(apolloNamespace);
                    stype = config.getSourceType();
                    if (!stype.name().equals("None")) {
                        break;
                    }
                }
            } else {
                config = ConfigService.getConfig(apolloNamespace);
                stype = config.getSourceType();
            }

            if(stype.name().equals("None") || stype == null) {
                log.error("getSourceType = {} is error, please check apollo server, or check apollo properties", stype.name());
                ((ConfigurableApplicationContext)this.appContext).close();
            }

            if (config != null) {
                config.addChangeListener(this);
            }
        }
    }

    public boolean isConnected() {
        if (!this.appID.equals("") && !this.apolloMeta.equals("")) {
            return true;
        }
        return false;
    }

    // @ApolloConfigChangeListener("gateway.yml")
    // public void listenApolloChange(ConfigChangeEvent changeEvent) {

    // }

    @Override
    public void onChange(ConfigChangeEvent changeEvent) {
        ConfigRefresh change = (ConfigRefresh) appContext.getBean(ConfigRefresh.class);
        change.notifyChanged();
    }

    /**
     * @return Config return the config
     */
    public Config getConfig() {
        return config;
    }

    /**
     * @param config the config to set
     */
    public void setConfig(Config config) {
        this.config = config;
    }

    /**
     * @return String return the apolloMeta
     */
    public String getApolloMeta() {
        return apolloMeta;
    }

    /**
     * @param apolloMeta the apolloMeta to set
     */
    public void setApolloMeta(String apolloMeta) {
        this.apolloMeta = apolloMeta;
    }

    /**
     * @return String return the appID
     */
    public String getAppID() {
        return appID;
    }

    /**
     * @param appID the appID to set
     */
    public void setAppID(String appID) {
        this.appID = appID;
    }

}